import {
  Controller,
  Get,
  Post,
  Body,
  Patch,
  Param,
  Delete,
  Query,
  NotFoundException,
  BadRequestException
} from '@nestjs/common';
import { ProductsService } from './products.service';
import { CreateProductDto } from './dto/create-product.dto';
import { UpdateProductDto } from './dto/update-product.dto';
import { UpdateProductSchema } from './dto/product.schema';
import { ZodError } from 'zod';
import { ResponseProductsDto } from './dto/product-response.dto';
import { Origin, Unit_Measure } from '../common/enums';
import { ApiQuery } from '@nestjs/swagger';

@Controller('products')
export class ProductsController {
  constructor(private readonly productsService: ProductsService) {}

  @Post()
  async create(
    @Body() createProductDto: CreateProductDto
  ): Promise<{ return: boolean; message: string }> {
    const errors = [];

    // Validação manual de campos
    if (!createProductDto.description) {
      errors.push({ field: 'description', message: 'Description is required' });
    }

    if (!createProductDto.category_id) {
      errors.push({ field: 'category_id', message: 'Category ID is required' });
    }

    if (!createProductDto.origin) {
      errors.push({ field: 'origin', message: 'Origin is required' });
    }

    // Verificando se a origem é válida
    if (
      createProductDto.origin !== Origin.RAW_MATERIAL &&
      createProductDto.origin !== Origin.MADE
    ) {
      errors.push({
        field: 'origin',
        message: 'Invalid origin. Must be RAW_MATERIAL or MADE'
      });
    }

    // Se houver erros, retorne com a mensagem de erro
    if (errors.length > 0) {
      return { return: false, message: JSON.stringify(errors) };
    }

    // Se a origem for RAW_MATERIAL ou MADE, chama a função correspondente
    if (createProductDto.origin === Origin.RAW_MATERIAL) {
      return this.createRaw(createProductDto);
    }

    if (createProductDto.origin === Origin.MADE) {
      return this.createMade(createProductDto);
    }

    return { return: false, message: 'Unexpected error: Invalid origin' };
  }
  // async create(
  //   @Body() createProductDto: CreateProductDto
  // ): Promise<ResponseProductsDto> {
  //   const errors = [];
  //   // Validando a origem
  //   if (!createProductDto.origin) {
  //     errors.push('Origin is required');
  //   }

  //   if (createProductDto.origin !== Origin.RAW_MATERIAL && createProductDto.origin !== Origin.MADE) {
  //     errors.push('Invalid origin');
  //   }

  //   // Se a origem for RAW_MATERIAL ou MADE, chama a função correspondente
  //   if (createProductDto.origin === Origin.RAW_MATERIAL) {
  //     return this.createRaw(createProductDto);
  //   }
  //   // else {
  //   //   return this.createMade(createProductDto);
  //   // }

  //   const descriptionValidation =
  //     CreateProductSchema.shape.description.safeParse(
  //       createProductDto.description
  //     );
  //   if (!descriptionValidation.success) {
  //     errors.push({
  //       field: 'description',
  //       message: descriptionValidation.error.errors[0].message
  //     });
  //   }

  //   const codeValidation = CreateProductSchema.shape.code.safeParse(
  //     createProductDto.code
  //   );
  //   if (!codeValidation.success) {
  //     errors.push({
  //       field: 'code',
  //       message: codeValidation.error.errors[0].message
  //     });
  //   }

  //   const skuValidation = CreateProductSchema.shape.sku.safeParse(
  //     createProductDto.sku
  //   );
  //   if (!skuValidation.success) {
  //     errors.push({
  //       field: 'sku',
  //       message: skuValidation.error.errors[0].message
  //     });
  //   }

  //   const originValidation = CreateProductSchema.shape.origin.safeParse(
  //     createProductDto.origin
  //   );
  //   if (!originValidation.success) {
  //     errors.push({
  //       field: 'origin',
  //       message: originValidation.error.errors[0].message
  //     });
  //   }

  //   const unitMeasureValidation =
  //     CreateProductSchema.shape.unit_measure.safeParse(
  //       createProductDto.unit_measure
  //     );
  //   if (!unitMeasureValidation.success) {
  //     errors.push({
  //       field: 'unit_measure',
  //       message: unitMeasureValidation.error.errors[0].message
  //     });
  //   }

  //   const categoryIdValidation =
  //     CreateProductSchema.shape.category_id.safeParse(
  //       createProductDto.category_id
  //     );
  //   if (!categoryIdValidation.success) {
  //     errors.push({
  //       field: 'category_id',
  //       message: categoryIdValidation.error.errors[0].message
  //     });
  //   }

  //   const groupIdValidation = CreateProductSchema.shape.group_id.safeParse(
  //     createProductDto.group_id
  //   );
  //   if (!groupIdValidation.success) {
  //     errors.push({
  //       field: 'group_id',
  //       message: groupIdValidation.error.errors[0].message
  //     });
  //   }

  //   const supplierIdValidation =
  //     CreateProductSchema.shape.supplier_id.safeParse(
  //       createProductDto.supplier_id
  //     );
  //   if (!supplierIdValidation.success) {
  //     errors.push({
  //       field: 'supplier_id',
  //       message: supplierIdValidation.error.errors[0].message
  //     });
  //   }

  //   const nutritionalInfoValidation =
  //     CreateProductSchema.shape.nutritional_info.safeParse(
  //       createProductDto.nutritional_info
  //     );
  //   if (!nutritionalInfoValidation.success) {
  //     errors.push({
  //       field: 'nutritional_info',
  //       message: nutritionalInfoValidation.error.errors[0].message
  //     });
  //   }

  //   if (errors.length > 0) {
  //     throw new BadRequestException(errors);
  //   }

  //   const matchedProducts = await this.productsService.matchProductByData(
  //     createProductDto.code,
  //     createProductDto.description,
  //     createProductDto.sku
  //   );

  //   if ((await matchedProducts).length > 0) {
  //     const existingProduct = matchedProducts[0];

  //     if (!existingProduct.active) {
  //       throw new BadRequestException(
  //         `Product already exists but is not active. Activate and update it: ${JSON.stringify(
  //           existingProduct
  //         )}`
  //       );
  //     }

  //     throw new BadRequestException(
  //       'Product already exists. Try update it instead'
  //     );
  //   }

  //   const createdProduct = await this.productsService.create(createProductDto);
  //   return {
  //     id: createdProduct.id,
  //     description: createdProduct.description,
  //     code: createdProduct.code,
  //     sku: createdProduct.sku,
  //     origin: createdProduct.origin as Origin,
  //     unit_measure: createdProduct.unit_measure as Unit_Measure,
  //     category_id: createdProduct.category_id,
  //     group_id: createdProduct.group_id,
  //     supplier_id: createdProduct.supplier_id,
  //     nutritional_info:
  //       typeof createdProduct.nutritional_info === 'string'
  //         ? JSON.parse(createdProduct.nutritional_info)
  //         : createdProduct.nutritional_info,
  //     active: createdProduct.active,
  //     created_at: createdProduct.created_at,
  //     updated_at: createdProduct.updated_at
  //   };
  // }

  @Post('raw')
  async createRaw(
    @Body() createProductDto: CreateProductDto
  ): Promise<{ return: boolean; message: string }> {
    try {
      const errors = [];
      // const result=await this.productsService.createRawMaterial(createProductDto);

      // Validação manual de campos
      if (!createProductDto.description) {
        errors.push({
          field: 'description',
          message: 'Description is required'
        });
      }

      if (!createProductDto.category_id) {
        errors.push({
          field: 'category_id',
          message: 'Category ID is required'
        });
      }

      // Se houver erros, retorne com a mensagem de erro
      if (errors.length > 0) {
        return { return: false, message: JSON.stringify(errors) };
      }

      // Cria uma nova instância de DTO com a origem modificada
      const newProductDto = {
        ...createProductDto,
        origin: Origin.RAW_MATERIAL
      };

      // Chama o serviço de criação de produto
      await this.productsService.create(newProductDto);

      return { return: true, message: 'Raw material created successfully' };
    } catch (error: unknown) {
      const errorMessage =
        error instanceof Error && error.message
          ? error.message.split('\n')[1] // Retorna apenas a primeira linha do erro
          : 'Unknown error';

      return {
        return: false,
        message: `Error creating raw material product: ${errorMessage}`
      };
    }
  }
  // async createRaw(
  //   @Body() createProductDto: CreateProductDto
  // ): Promise<{ return: boolean; message: string }> {
  //   try {
  //     // Cria uma nova instância de DTO com a origem modificada
  //     const newProductDto = { ...createProductDto, origin: Origin.RAW_MATERIAL };

  //     // Chama o serviço de criação de produto com os dados de produto final
  //     await this.productsService.create(newProductDto);

  //     return { return: true, message: 'Raw material created successfully' };
  //   } catch (error) {
  //     return { return: false, message: 'Error creating raw material product: unknown error' };
  //   }
  // }

  @Post('made')
  async createMade(
    @Body() createProductDto: CreateProductDto
  ): Promise<{ return: boolean; message: string }> {
    try {
      const errors = [];
      // const result=await this.productsService.createMade(createProductDto);

      // Validação manual de campos
      if (!createProductDto.description) {
        errors.push({
          field: 'description',
          message: 'Description is required'
        });
      }

      if (!createProductDto.category_id) {
        errors.push({
          field: 'category_id',
          message: 'Category ID is required'
        });
      }

      // Se houver erros, retorne com a mensagem de erro
      if (errors.length > 0) {
        return { return: false, message: JSON.stringify(errors) };
      }

      // Cria uma nova instância de DTO com a origem modificada
      const newProductDto = { ...createProductDto, origin: Origin.MADE };

      // Chama o serviço de criação de produto
      await this.productsService.create(newProductDto);

      return { return: true, message: 'Made product created successfully' };
    } catch (error: unknown) {
      const errorMessage =
        error instanceof Error && error.message
          ? error.message.split('\n')[1] // Retorna apenas a primeira linha do erro
          : 'Unknown error';

      return {
        return: false,
        message: `Error creating made product: ${errorMessage}`
      };
    }
  }
  // async createMade(
  //   @Body() createProductDto: CreateProductDto
  // ): Promise<{ return: boolean; message: string }> {
  //   try {
  //     // Cria uma nova instância de DTO com a origem modificada
  //     const newProductDto = { ...createProductDto, origin: Origin.MADE };

  //     // Chama o serviço de criação de produto com os dados de produto final
  //     await this.productsService.create(newProductDto);

  //     return { return: true, message: 'Made product created successfully' };
  //   } catch (error) {
  //     return { return: false, message: 'Error creating made product: unknown error'};
  //   }
  // }

  @Get()
  @ApiQuery({
    name: 'orderBy',
    required: false,
    description:
      'Field to order by. Valid fields: id, description, code, sku, origin, category_id, group_id, supplier_id',
    enum: [
      'id',
      'description',
      'code',
      'sku',
      'origin',
      'category_id',
      'supplier_id',
      'group_id'
    ]
  })
  async findAll(
    @Query('orderBy') orderBy: string = 'id'
  ): Promise<ResponseProductsDto[]> {
    const validOrderFields = [
      'id',
      'description',
      'code',
      'sku',
      'origin',
      'category_id',
      'group_id',
      'supplier_id'
    ];

    if (!validOrderFields.includes(orderBy)) {
      throw new BadRequestException(`Invalid order field: ${orderBy}`);
    }

    const products = await this.productsService.findAll(orderBy);
    return products.map(product => ({
      id: product.id,
      description: product.description,
      code: product.code,
      sku: product.sku,
      origin: product.origin as Origin,
      unit_measure: product.unit_measure as Unit_Measure,
      category_id: product.category_id,
      group_id: product.group_id,
      supplier_id: product.supplier_id,
      nutritional_info:
        typeof product.nutritional_info === 'string'
          ? JSON.parse(product.nutritional_info)
          : product.nutritional_info,
      active: product.active,
      created_at: product.created_at,
      updated_at: product.updated_at
    }));
  }

  @Get('made')
  @ApiQuery({
    name: 'orderBy',
    required: false,
    description:
      'Field to order by. Valid fields: id, description, code, sku, category_id, group_id, supplier_id',
    enum: [
      'id',
      'description',
      'code',
      'sku',
      'category_id',
      'group_id',
      'supplier_id'
    ]
  })
  async findMade(
    @Query('orderBy') orderBy: string = 'id'
  ): Promise<ResponseProductsDto[]> {
    const validOrderFields = [
      'id',
      'description',
      'code',
      'sku',
      'category_id',
      'group_id',
      'supplier_id'
    ];

    if (!validOrderFields.includes(orderBy)) {
      throw new BadRequestException(`Invalid order field: ${orderBy}`);
    }
    const origin = Origin.MADE;
    const products = await this.productsService.findByOrigin(orderBy, origin);
    return products.map(product => ({
      id: product.id,
      description: product.description,
      code: product.code,
      sku: product.sku,
      origin: product.origin as Origin,
      unit_measure: product.unit_measure as Unit_Measure,
      category_id: product.category_id,
      group_id: product.group_id,
      supplier_id: product.supplier_id,
      nutritional_info:
        typeof product.nutritional_info === 'string'
          ? JSON.parse(product.nutritional_info)
          : product.nutritional_info,
      active: product.active,
      created_at: product.created_at,
      updated_at: product.updated_at
    }));
  }

  @Get('raw-material')
  @ApiQuery({
    name: 'orderBy',
    required: false,
    description:
      'Field to order by. Valid fields: id, description, code, sku, category_id, group_id, supplier_id',
    enum: [
      'id',
      'description',
      'code',
      'sku',
      'category_id',
      'group_id',
      'supplier_id'
    ]
  })
  async findRaw(
    @Query('orderBy') orderBy: string = 'id'
  ): Promise<ResponseProductsDto[]> {
    const validOrderFields = [
      'id',
      'description',
      'code',
      'sku',
      'category_id',
      'group_id',
      'supplier_id'
    ];

    if (!validOrderFields.includes(orderBy)) {
      throw new BadRequestException(`Invalid order field: ${orderBy}`);
    }
    const origin = Origin.RAW_MATERIAL;
    const products = await this.productsService.findByOrigin(orderBy, origin);
    return products.map(product => ({
      id: product.id,
      description: product.description,
      code: product.code,
      sku: product.sku,
      origin: product.origin as Origin,
      unit_measure: product.unit_measure as Unit_Measure,
      category_id: product.category_id,
      group_id: product.group_id,
      supplier_id: product.supplier_id,
      nutritional_info:
        typeof product.nutritional_info === 'string'
          ? JSON.parse(product.nutritional_info)
          : product.nutritional_info,
      active: product.active,
      created_at: product.created_at,
      updated_at: product.updated_at
    }));
  }

  @Get(':id')
  findById(@Param('id') id: string): Promise<ResponseProductsDto> {
    const idNumber = +id;
    if (isNaN(idNumber)) {
      throw new BadRequestException('Invalid ID format');
    }

    const product = this.productsService.findById(idNumber);

    return product.then(product => ({
      id: product.id,
      description: product.description,
      code: product.code,
      sku: product.sku,
      origin: product.origin as Origin,
      unit_measure: product.unit_measure as Unit_Measure,
      category_id: product.category_id,
      group_id: product.group_id,
      supplier_id: product.supplier_id,
      nutritional_info:
        typeof product.nutritional_info === 'string'
          ? JSON.parse(product.nutritional_info)
          : product.nutritional_info,
      active: product.active,
      created_at: product.created_at,
      updated_at: product.updated_at
    }));
  }

  @Patch(':id')
  async update(
    @Param('id') id: string,
    @Body() updateProductDto: UpdateProductDto,
    @Query() queryParams: Record<string, string>
  ): Promise<ResponseProductsDto> {
    const idNumber = +id;
    if (isNaN(idNumber)) {
      throw new BadRequestException('Invalid ID format');
    }

    const existingProduct = await this.productsService.findById(idNumber);
    if (!existingProduct) {
      throw new NotFoundException(`Product with ID ${idNumber} not found`);
    }

    if (!existingProduct.active) {
      throw new BadRequestException(`Product ID ${idNumber} is not active`);
    }

    const allowedFields = Object.keys(UpdateProductSchema.shape);
    const fieldsToUpdate = Object.keys(queryParams);

    if (fieldsToUpdate.length === 0) {
      throw new BadRequestException('No fields provided to update');
    }

    const updateData: Partial<UpdateProductDto> = {};

    for (const field of fieldsToUpdate) {
      if (!allowedFields.includes(field)) {
        throw new BadRequestException(`Invalid field: ${field}`);
      }

      const value = queryParams[field];

      if (['category_id', 'group_id', 'supplier_id'].includes(field)) {
        const numericValue = parseInt(value, 10);
        if (isNaN(numericValue)) {
          throw new BadRequestException(
            `Invalid number format for field: ${field}`
          );
        }
        updateData[field] = numericValue;
      } else {
        if (value !== undefined) {
          updateData[field] = value;
        }
      }
    }

    const validation = UpdateProductSchema.safeParse(updateData);

    if (!validation.success) {
      const zodError = validation.error as ZodError;
      const firstError = zodError.errors[0];
      throw new BadRequestException(
        `${firstError.path[0]} is invalid: ${firstError.message}`
      );
    }

    const updatedProduct = await this.productsService.update(
      idNumber,
      updateData as UpdateProductDto
    );

    return {
      id: updatedProduct.id,
      description: updatedProduct.description,
      code: updatedProduct.code,
      sku: updatedProduct.sku,
      origin: updatedProduct.origin as Origin,
      unit_measure: updatedProduct.unit_measure as Unit_Measure,
      category_id: updatedProduct.category_id,
      group_id: updatedProduct.group_id,
      supplier_id: updatedProduct.supplier_id,
      nutritional_info:
        typeof updatedProduct.nutritional_info === 'string'
          ? JSON.parse(updatedProduct.nutritional_info)
          : updatedProduct.nutritional_info,
      active: updatedProduct.active,
      created_at: updatedProduct.created_at,
      updated_at: updatedProduct.updated_at
    };
  }

  @Delete(':id')
  async remove(@Param('id') id: string) {
    const idNumber = +id;

    if (isNaN(idNumber)) {
      throw new BadRequestException('Invalid ID format');
    }
    await this.productsService.remove(idNumber);
    return { message: `Product ID ${idNumber} deleted successfully` };
  }

  @Post('activate/:id')
  async activate(@Param('id') id: string) {
    const idNumber = +id;
    if (isNaN(idNumber)) {
      throw new BadRequestException('Invalid ID format');
    }
    await this.productsService.reactivateProduct(idNumber);

    return { message: `Product ID ${id} activated successfully` };
  }
}
